<?php
namespace DDTrace {
class Type
{
    const CACHE = 'cache';
    const HTTP_CLIENT = 'http';
    const WEB_SERVLET = 'web';
    const CLI = 'cli';
    const SQL = 'sql';
    const MESSAGE_CONSUMER = 'queue';
    const MESSAGE_PRODUCER = 'queue';
    const CASSANDRA = 'cassandra';
    const ELASTICSEARCH = 'elasticsearch';
    const MEMCACHED = 'memcached';
    const MONGO = 'mongodb';
    const REDIS = 'redis';
}
}
namespace DDTrace {
class Tag
{
    const ENV = 'env';
    const SPAN_TYPE = 'span.type';
    const SPAN_KIND = 'span.kind';
    const SPAN_KIND_VALUE_SERVER = 'server';
    const SPAN_KIND_VALUE_CLIENT = 'client';
    const SPAN_KIND_VALUE_PRODUCER = 'producer';
    const SPAN_KIND_VALUE_CONSUMER = 'consumer';
    const SPAN_KIND_VALUE_INTERNAL = 'internal';
    const COMPONENT = 'component';
    const SERVICE_NAME = 'service.name';
    const MANUAL_KEEP = 'manual.keep';
    const MANUAL_DROP = 'manual.drop';
    const PID = 'process_id';
    const RESOURCE_NAME = 'resource.name';
    const DB_STATEMENT = 'sql.query';
    const ERROR = 'error';
    const ERROR_MSG = 'error.message';
    const ERROR_TYPE = 'error.type';
    const ERROR_STACK = 'error.stack';
    const HTTP_METHOD = 'http.method';
    const HTTP_ROUTE = 'http.route';
    const HTTP_STATUS_CODE = 'http.status_code';
    const HTTP_URL = 'http.url';
    const HTTP_VERSION = 'http.version';
    const LOG_EVENT = 'event';
    const LOG_ERROR = 'error';
    const LOG_ERROR_OBJECT = 'error.object';
    const LOG_MESSAGE = 'message';
    const LOG_STACK = 'stack';
    const NETWORK_DESTINATION_NAME = 'network.destination.name';
    const TARGET_HOST = 'out.host';
    const TARGET_PORT = 'out.port';
    const BYTES_OUT = 'net.out.bytes';
    const ANALYTICS_KEY = '_dd1.sr.eausr';
    const HOSTNAME = '_dd.hostname';
    const ORIGIN = '_dd.origin';
    const VERSION = 'version';
    const SERVICE_VERSION = 'service.version';
    const ELASTICSEARCH_BODY = 'elasticsearch.body';
    const ELASTICSEARCH_METHOD = 'elasticsearch.method';
    const ELASTICSEARCH_PARAMS = 'elasticsearch.params';
    const ELASTICSEARCH_URL = 'elasticsearch.url';
    const DB_NAME = 'db.name';
    const DB_CHARSET = 'db.charset';
    const DB_INSTANCE = 'db.instance';
    const DB_TYPE = 'db.type';
    const DB_SYSTEM = 'db.system';
    const DB_ROW_COUNT = 'db.row_count';
    const DB_STMT = 'db.statement';
    const DB_USER = 'db.user';
    const LARAVELQ_ATTEMPTS = 'messaging.laravel.attempts';
    const LARAVELQ_BATCH_ID = 'messaging.laravel.batch_id';
    const LARAVELQ_CONNECTION = 'messaging.laravel.connection';
    const LARAVELQ_MAX_TRIES = 'messaging.laravel.max_tries';
    const LARAVELQ_NAME = 'messaging.laravel.name';
    const LARAVELQ_TIMEOUT = 'messaging.laravel.timeout';
    const MONGODB_BSON_ID = 'mongodb.bson.id';
    const MONGODB_COLLECTION = 'mongodb.collection';
    const MONGODB_DATABASE = 'mongodb.db';
    const MONGODB_PROFILING_LEVEL = 'mongodb.profiling_level';
    const MONGODB_READ_PREFERENCE = 'mongodb.read_preference';
    const MONGODB_SERVER = 'mongodb.server';
    const MONGODB_TIMEOUT = 'mongodb.timeout';
    const MONGODB_QUERY = 'mongodb.query';
    const REDIS_RAW_COMMAND = 'redis.raw_command';
    const MQ_SYSTEM = 'messaging.system';
    const MQ_DESTINATION = 'messaging.destination';
    const MQ_DESTINATION_KIND = 'messaging.destination_kind';
    const MQ_TEMP_DESTINATION = 'messaging.temp_destination';
    const MQ_PROTOCOL = 'messaging.protocol';
    const MQ_PROTOCOL_VERSION = 'messaging.protocol_version';
    const MQ_URL = 'messaging.url';
    const MQ_MESSAGE_ID = 'messaging.message_id';
    const MQ_CONVERSATION_ID = 'messaging.conversation_id';
    const MQ_MESSAGE_PAYLOAD_SIZE = 'messaging.message_payload_size_bytes';
    const MQ_OPERATION = 'messaging.operation';
    const MQ_CONSUMER_ID = 'messaging.consumer_id';
    const RABBITMQ_DELIVERY_MODE = 'messaging.rabbitmq.delivery_mode';
    const RABBITMQ_EXCHANGE = 'messaging.rabbitmq.exchange';
    const RABBITMQ_ROUTING_KEY = 'messaging.rabbitmq.routing_key';
}
}
namespace DDTrace\Log {
interface LoggerInterface
{
    public function debug($message, array $context = array());
    public function warning($message, array $context = array());
    public function error($message, array $context = array());
    public function isLevelActive($level);
}
}
namespace DDTrace\Log {
trait InterpolateTrait
{
    public function interpolate($message, array $context = array())
    {
        $replace = array();
        foreach ($context as $key => $val) {
            if (is_array($val)) {
                $val = '{Array of size ' . count($val) . '}';
            } elseif (is_object($val) && !method_exists($val, '__toString')) {
                $val = '{Object of type ' . get_class($val) . '}';
            }
            $replace['{' . $key . '}'] = $val;
        }
        return strtr($message, $replace);
    }
}
}
namespace DDTrace\Log {
final class LogLevel
{
    const EMERGENCY = 'emergency';
    const ALERT = 'alert';
    const CRITICAL = 'critical';
    const ERROR = 'error';
    const WARNING = 'warning';
    const NOTICE = 'notice';
    const INFO = 'info';
    const DEBUG = 'debug';
    public static function all()
    {
        return array(self::EMERGENCY, self::ALERT, self::CRITICAL, self::ERROR, self::WARNING, self::NOTICE, self::INFO, self::DEBUG);
    }
}
}
namespace DDTrace\Log {
final class Logger
{
    private static $logger;
    public static function set(LoggerInterface $logger)
    {
        self::$logger = $logger;
    }
    public static function get()
    {
        if (self::$logger === null) {
            self::$logger = self::getBoolIni('datadog.trace.debug') ? new ErrorLogLogger(LogLevel::DEBUG) : new NullLogger(LogLevel::EMERGENCY);
        }
        return self::$logger;
    }
    private static function getBoolIni($iniName)
    {
        $ini = ini_get($iniName);
        return $ini === 'on' || $ini === 'yes' || $ini === 'true' || (int) $ini;
    }
    public static function reset()
    {
        self::$logger = null;
    }
}
}
namespace DDTrace\Log {
abstract class AbstractLogger implements LoggerInterface
{
    private $enabledLevels = array();
    public function __construct($level)
    {
        $level = trim(strtolower($level));
        # all() have all levels ordered from highest to lowest
        # all preceding levels to given $level will be marked as enabled
        $enabled = false;
        foreach (array_reverse(LogLevel::all()) as $knownLevel) {
            $enabled = $enabled || $level === $knownLevel;
            $this->enabledLevels[$knownLevel] = $enabled;
        }
    }
    public function isLevelActive($level)
    {
        return (bool) $this->enabledLevels[$level];
    }
}
}
namespace DDTrace\Log {
class ErrorLogLogger extends AbstractLogger
{
    use InterpolateTrait;
    public function debug($message, array $context = array())
    {
        $this->emit(LogLevel::DEBUG, $message, $context);
    }
    public function warning($message, array $context = array())
    {
        $this->emit(LogLevel::WARNING, $message, $context);
    }
    public function error($message, array $context = array())
    {
        $this->emit(LogLevel::ERROR, $message, $context);
    }
    private function emit($level, $message, array $context = array())
    {
        if (!$this->isLevelActive($level)) {
            return;
        }
        $interpolatedMessage = $this->interpolate($message, $context);
        $date = date(\DateTime::ATOM);
        error_log("[{$date}] [ddtrace] [{$level}] - {$interpolatedMessage}");
    }
}
}
namespace DDTrace\Log {
final class NullLogger extends AbstractLogger
{
    public function debug($message, array $context = array())
    {
    }
    public function warning($message, array $context = array())
    {
    }
    public function error($message, array $context = array())
    {
    }
    public function isLevelActive($level)
    {
        return false;
    }
}
}
namespace DDTrace\Log {
trait LoggingTrait
{
    protected static function logDebug($message, array $context = array())
    {
        Logger::get()->debug($message, $context);
    }
    protected static function logWarning($message, array $context = array())
    {
        Logger::get()->warning($message, $context);
    }
    protected static function logError($message, array $context = array())
    {
        Logger::get()->error($message, $context);
    }
    protected static function isLogDebugActive()
    {
        return Logger::get()->isLevelActive(LogLevel::DEBUG);
    }
}
}
namespace DDTrace\Http {
use DDTrace\Obfuscation\WildcardToRegex;
class Urls
{
    private static $defaultPatterns = array('<(/)([0-9a-f]{8}-?[0-9a-f]{4}-?[1-5][0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12})(/|$)>i', '<(/)([0-9a-f]{8,128})(/|$)>i', '<(/)([0-9]+)(/|$)>');
    private $replacementPatterns = array();
    public function __construct(array $patternsWithWildcards = array())
    {
        foreach ($patternsWithWildcards as $pattern) {
            $this->replacementPatterns[] = WildcardToRegex::convert($pattern);
        }
    }
    public static function sanitize($url, $dropUserInfo = false)
    {
        $url = (string) $url;
        $userinfoPattern = '[a-zA-Z0-9\\-._~!$&\'()*+,;=%?]+';
        $sanitizedUserinfo = preg_replace(array("/{$userinfoPattern}:@/", "/{$userinfoPattern}:{$userinfoPattern}@/"), array($dropUserInfo ? '' : '<sanitized>:@', $dropUserInfo ? '' : '<sanitized>:<sanitized>@'), $url);
        return \str_replace('<sanitized>', '?', strstr($sanitizedUserinfo, '?', true) ?: $sanitizedUserinfo);
    }
    public static function hostname($url)
    {
        $url = self::sanitize($url, true);
        $unparsableUrl = 'unparsable-host';
        $parts = \parse_url($url);
        if (!$parts) {
            return $unparsableUrl;
        }
        if (isset($parts['host'])) {
            return $parts['host'];
        }
        if (empty($parts['path'])) {
            return $unparsableUrl;
        }
        $path = $parts['path'];
        if (\substr($path, 0, 1) === '/') {
            return 'unknown-host';
        }
        $pathFragments = \explode('/', $path);
        return $pathFragments[0];
    }
    public static function hostnameForTag($url)
    {
        $url = \trim((string) $url);
        $knownUnixProtocols = array('uds', 'unix', 'http+unix', 'https+unix');
        foreach ($knownUnixProtocols as $protocol) {
            $length = \strlen($protocol);
            if ($protocol . '://' === \substr($url, 0, $length + 3)) {
                return 'socket-' . Urls::normalizeFileSystemPath(\substr($url, $length + 3));
            }
        }
        return 'host-' . self::hostname($url);
    }
    private static function normalizeFileSystemPath($url)
    {
        return \trim(\preg_replace('/[^0-9a-zA-Z\\.]+/', '-', \trim($url)), '-');
    }
    public function normalize($url)
    {
        $url = self::sanitize($url);
        foreach ($this->replacementPatterns as $regexReplacement) {
            list($regex, $replacement) = $regexReplacement;
            $replacedCount = 0;
            $url = preg_replace($regex, $replacement, $url, -1, $replacedCount);
            if ($replacedCount > 0) {
                return $url;
            }
        }
        return preg_replace(self::$defaultPatterns, '$1?$3', $url);
    }
}
}
